@@ -1,3 +1,4 @@
+import dataclasses
 import json
 import os.path
 import pathlib
@@ -6,10 +7,10 @@ import xml.etree.ElementTree as ET
 
 from eth_utils import event_abi_to_log_topic, encode_hex, function_abi_to_4byte_selector
 
-from bdbt.content import Contract
 from bdbt.ethereum.abi.abi_data_type import ABIEventSchema, ABICallSchema
 from bdbt.ethereum.abi.provider.hive_object_inspector_type_provider import HiveObjectInspectorTypeProvider
 from bdbt.ethereum.dbt.dbt_code_generator import DbtCodeGenerator
+from bdbt.global_type import Contract
 
 event_clazz_template = """package io.iftech.sparkudf.hive;
 
@@ -73,7 +74,7 @@ empty_event_dbt_model_sql_template = """{{
     )
 }}
 
-select /* REPARTITION({{MODEL_REPARTITION_COUNT}}) */
+select /*+ REPARTITION({{MODEL_REPARTITION_COUNT}}) */
     block_number as evt_block_number,
     block_timestamp as evt_block_time,
     log_index as evt_index,
@@ -81,10 +82,7 @@ select /* REPARTITION({{MODEL_REPARTITION_COUNT}}) */
     address as contract_address,
     dt
 from {{ ref('stg_logs') }}
-where address = lower("{{CONTRACT_ADDRESS}}")
-and address_hash = abs(hash(lower("{{CONTRACT_ADDRESS}}"))) % 10
-and selector = "{{EVENT_SELECTOR}}"
-and selector_hash = abs(hash("{{EVENT_SELECTOR}}")) % 10
+where {{SELECT_CONDITION}}
 
 {% if is_incremental() %}
   and dt = '{{ var("dt") }}'
@@ -112,10 +110,7 @@ with base as (
         dt,
         {{UDF_NAME}}(unhex_data, topics_arr, '{{EVENT_ABI}}', '{{EVENT_NAME}}') as data
     from {{ ref('stg_logs') }}
-    where address = lower("{{CONTRACT_ADDRESS}}")
-    and address_hash = abs(hash(lower("{{CONTRACT_ADDRESS}}"))) % 10
-    and selector = "{{EVENT_SELECTOR}}"
-    and selector_hash = abs(hash("{{EVENT_SELECTOR}}")) % 10
+    where {{SELECT_CONDITION}}
 
     {% if is_incremental() %}
       and dt = '{{ var("dt") }}'
@@ -134,7 +129,7 @@ final as (
     from base
 )
 
-select /* REPARTITION({{MODEL_REPARTITION_COUNT}}) */ *
+select /*+ REPARTITION({{MODEL_REPARTITION_COUNT}}) */ *
 from final
 """
 
@@ -146,7 +141,7 @@ empty_call_dbt_model_sql_template = """{{
     )
 }}
 
-select /* REPARTITION({{MODEL_REPARTITION_COUNT}}) */
+select /*+ REPARTITION({{MODEL_REPARTITION_COUNT}}) */
     status==1 as call_success,
     block_number as call_block_number,
     block_timestamp as call_block_time,
@@ -155,10 +150,7 @@ select /* REPARTITION({{MODEL_REPARTITION_COUNT}}) */
     to_address as contract_address,
     dt
 from {{ ref('stg_traces') }}
-where to_address = lower("{{CONTRACT_ADDRESS}}")
-and address_hash = abs(hash(lower("{{CONTRACT_ADDRESS}}"))) % 10
-and selector = "{{CALL_SELECTOR}}"
-and selector_hash = abs(hash("{{CALL_SELECTOR}}")) % 10
+where {{SELECT_CONDITION}}
 
 {% if is_incremental() %}
   and dt = '{{ var("dt") }}'
@@ -187,10 +179,7 @@ with base as (
         dt,
         {{UDF_NAME}}(unhex_input, unhex_output, '{{CALL_ABI}}', '{{CALL_NAME}}') as data
     from {{ ref('stg_traces') }}
-    where to_address = lower("{{CONTRACT_ADDRESS}}")
-    and address_hash = abs(hash(lower("{{CONTRACT_ADDRESS}}"))) % 10
-    and selector = "{{CALL_SELECTOR}}"
-    and selector_hash = abs(hash("{{CALL_SELECTOR}}")) % 10
+    where {{SELECT_CONDITION}}
 
     {% if is_incremental() %}
       and dt = '{{ var("dt") }}'
@@ -211,7 +200,7 @@ final as (
     from base
 )
 
-select /* REPARTITION({{MODEL_REPARTITION_COUNT}}) */ *
+select /*+ REPARTITION({{MODEL_REPARTITION_COUNT}}) */ *
 from final
 """
 
@@ -233,18 +222,15 @@ class SparkDbtCodeGenerator(DbtCodeGenerator):
             version: str,
             event: ABIEventSchema
     ) -> None:
-        contract_name = contract['name']
-        contract_address = contract['address']
-        contract_materialize = contract['materialize']
+        contract_name = contract.name
+        contract_materialize = contract.materialize
 
         project_name = pathlib.Path(project_path).name
         filepath = os.path.join(project_path, self.evt_model_name(contract_name, event, project_name) + '.sql')
-        event_selector = encode_hex(event_abi_to_log_topic(event.raw_schema))
 
         if event.is_empty:
             content = event_dbt_model_sql_template \
-                .replace('{{CONTRACT_ADDRESS}}', contract_address) \
-                .replace('{{EVENT_SELECTOR}}', event_selector) \
+                .replace('{{SELECT_CONDITION}}', self._select_evt_condition(contract, event)) \
                 .replace('{{MODEL_ALIAS}}', self.evt_model_name(contract_name, event).lower()) \
                 .replace('{{MODEL_MATERIALIZED_CONFIG}}', self._materialized_config(contract_materialize)) \
                 .replace('{{MODEL_REPARTITION_COUNT}}', self._repartition_count(contract_materialize))
@@ -254,10 +240,9 @@ class SparkDbtCodeGenerator(DbtCodeGenerator):
                 .replace('{{UDF_NAME}}', clazz_name.lower()) \
                 .replace('{{CLASS_NAME}}', clazz_name) \
                 .replace('{{UDF_JAR_PATH}}', os.path.join(self.remote_workspace, self._jar_name(version))) \
-                .replace('{{EVENT_ABI}}', json.dumps(event.raw_schema)) \
+                .replace('{{EVENT_ABI}}', json.dumps(event.raw_schema.to_dict(omit_none=True))) \
                 .replace('{{EVENT_NAME}}', event.name) \
-                .replace('{{CONTRACT_ADDRESS}}', contract_address) \
-                .replace('{{EVENT_SELECTOR}}', event_selector) \
+                .replace('{{SELECT_CONDITION}}', self._select_evt_condition(contract, event)) \
                 .replace('{{MODEL_ALIAS}}', self.evt_model_name(contract_name, event).lower()) \
                 .replace('{{MODEL_MATERIALIZED_CONFIG}}', self._materialized_config(contract_materialize)) \
                 .replace('{{MODEL_REPARTITION_COUNT}}', self._repartition_count(contract_materialize))
@@ -271,18 +256,15 @@ class SparkDbtCodeGenerator(DbtCodeGenerator):
             version: str,
             call: ABICallSchema
     ) -> None:
-        contract_name = contract['name']
-        contract_address = contract['address']
-        contract_materialize = contract['materialize']
+        contract_name = contract.name
+        contract_materialize = contract.materialize
 
         project_name = pathlib.Path(project_path).name
         filepath = os.path.join(project_path, self.call_model_name(contract_name, call, project_name) + '.sql')
-        call_selector = encode_hex(encode_hex(function_abi_to_4byte_selector(call.raw_schema)))
 
         if call.is_empty:
             content = empty_call_dbt_model_sql_template \
-                .replace('{{CONTRACT_ADDRESS}}', contract_address) \
-                .replace('{{CALL_SELECTOR}}', call_selector) \
+                .replace('{{SELECT_CONDITION}}', self._select_call_condition(contract, call)) \
                 .replace('{{MODEL_ALIAS}}', self.call_model_name(contract_name, call).lower()) \
                 .replace('{{MODEL_MATERIALIZED_CONFIG}}', self._materialized_config(contract_materialize)) \
                 .replace('{{MODEL_REPARTITION_COUNT}}', self._repartition_count(contract_materialize))
@@ -292,10 +274,9 @@ class SparkDbtCodeGenerator(DbtCodeGenerator):
                 .replace('{{UDF_NAME}}', clazz_name.lower()) \
                 .replace('{{CLASS_NAME}}', clazz_name) \
                 .replace('{{UDF_JAR_PATH}}', os.path.join(self.remote_workspace, self._jar_name(version))) \
-                .replace('{{CALL_ABI}}', json.dumps(call.raw_schema)) \
+                .replace('{{CALL_ABI}}', json.dumps(call.raw_schema.to_dict(omit_none=True))) \
                 .replace('{{CALL_NAME}}', call.name) \
-                .replace('{{CONTRACT_ADDRESS}}', contract_address) \
-                .replace('{{CALL_SELECTOR}}', call_selector[0:10]) \
+                .replace('{{SELECT_CONDITION}}', self._select_call_condition(contract, call)) \
                 .replace('{{MODEL_ALIAS}}', self.call_model_name(contract_name, call).lower()) \
                 .replace('{{MODEL_MATERIALIZED_CONFIG}}', self._materialized_config(contract_materialize)) \
                 .replace('{{MODEL_REPARTITION_COUNT}}', self._repartition_count(contract_materialize))
@@ -423,9 +404,42 @@ class SparkDbtCodeGenerator(DbtCodeGenerator):
 
     @staticmethod
     def _repartition_count(materialize: str) -> str:
+        # the best partition size in the Spark is 100-200MB,
+        # combining the number of files and the file size,
+        # 50 is more suitable for the current data size (the largest table is probably 10G)
         if materialize == 'table':
-            return '1'
+            return '50'
         elif materialize == 'increment':
             return 'dt'
         else:
             raise ValueError(f'{materialize} isnt a supported materialized model.')
+
+    @staticmethod
+    def _select_call_condition(contract: Contract, call: ABICallSchema) -> str:
+        conditions = []
+        if contract.address:
+            conditions.append(
+                f"""to_address = lower("{contract.address}") and address_hash = abs(hash(lower("{contract.address}"))) % 10"""
+            )
+
+        selector = encode_hex(function_abi_to_4byte_selector(dataclasses.asdict(call.raw_schema)))[0:10]
+        conditions.append(
+            f"""selector = "{selector}" and selector_hash = abs(hash("{selector}")) % 10"""
+        )
+
+        return ' and '.join(conditions)
+
+    @staticmethod
+    def _select_evt_condition(contract: Contract, evt: ABIEventSchema) -> str:
+        conditions = []
+        if contract.address:
+            conditions.append(
+                f"""address = lower("{contract.address}") and address_hash = abs(hash(lower("{contract.address}"))) % 10"""
+            )
+
+        selector = encode_hex(event_abi_to_log_topic(dataclasses.asdict(evt.raw_schema)))
+        conditions.append(
+            f"""selector = "{selector}" and selector_hash = abs(hash("{selector}")) % 10"""
+        )
+
+        return ' and '.join(conditions)
